@@ -6,6 +6,7 @@ import (
 	"io/ioutil"
 	"os"
 	"os/signal"
+	"path/filepath"
 	"runtime"
 	"strings"
 	"syscall"
@@ -14,12 +15,14 @@ import (
 
 	"github.com/sirupsen/logrus"
 	"github.com/urfave/cli"
+	"gopkg.in/fsnotify/fsnotify.v1"
 
+	hocontroller "github.com/ovn-org/ovn-kubernetes/go-controller/hybrid-overlay/pkg/controller"
 	ovncluster "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/cluster"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/factory"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/ovn"
-	util "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 
 	kexec "k8s.io/utils/exec"
 )
@@ -58,6 +61,7 @@ func getFlagsByCategory() map[string][]cli.Flag {
 	m["OVN Northbound DB Options"] = config.OvnNBFlags
 	m["OVN Southbound DB Options"] = config.OvnSBFlags
 	m["OVN Gateway Options"] = config.OVNGatewayFlags
+	m["Master HA Options"] = config.MasterHAFlags
 
 	return m
 }
@@ -69,10 +73,8 @@ func printOvnKubeHelp(out io.Writer, templ string, data interface{}, customFunc
 		"upper":              strings.ToUpper,
 		"getFlagsByCategory": getFlagsByCategory,
 	}
-	if customFunc != nil {
-		for key, value := range customFunc {
-			funcMap[key] = value
-		}
+	for key, value := range customFunc {
+		funcMap[key] = value
 	}
 
 	w := tabwriter.NewWriter(out, 1, 8, 2, ' ', 0)
@@ -96,6 +98,13 @@ func main() {
 	c.Flags = append(c.Flags, config.OvnNBFlags...)
 	c.Flags = append(c.Flags, config.OvnSBFlags...)
 	c.Flags = append(c.Flags, config.OVNGatewayFlags...)
+	c.Flags = append(c.Flags, config.MasterHAFlags...)
+	c.Flags = append(c.Flags, hocontroller.GetHybridOverlayCLIFlags([]cli.Flag{
+		cli.BoolFlag{
+			Name:  "enable-hybrid-overlay",
+			Usage: "Enables hybrid overlay operation (requires --init-master and/or --init-node)",
+		}})...)
+
 	c.Action = func(c *cli.Context) error {
 		return runOvnKube(c)
 	}
@@ -136,8 +145,7 @@ func setupPIDFile(pidfile string) error {
 		// get the pid and see if it exists
 		pid, err := ioutil.ReadFile(pidfile)
 		if err != nil {
-			logrus.Errorf("pidfile %s exists but can't be read", pidfile)
-			return err
+			return fmt.Errorf("pidfile %s exists but can't be read: %v", pidfile, err)
 		}
 		_, err1 := os.Stat("/proc/" + string(pid[:]) + "/cmdline")
 		if os.IsNotExist(err1) {
@@ -146,8 +154,7 @@ func setupPIDFile(pidfile string) error {
 				logrus.Errorf("failed to write pidfile %s (%v). Ignoring..", pidfile, err)
 			}
 		} else {
-			logrus.Errorf("pidfile %s exists and ovnkube is running", pidfile)
-			os.Exit(1)
+			return fmt.Errorf("pidfile %s exists and ovnkube is running", pidfile)
 		}
 	}
 
@@ -155,89 +162,183 @@ func setupPIDFile(pidfile string) error {
 }
 
 func runOvnKube(ctx *cli.Context) error {
+	pidfile := ctx.String("pidfile")
+	if pidfile != "" {
+		defer delPidfile(pidfile)
+		if err := setupPIDFile(pidfile); err != nil {
+			return err
+		}
+	}
+
 	exec := kexec.New()
-	_, err := config.InitConfig(ctx, exec, nil)
+	configFile, err := config.InitConfig(ctx, exec, nil)
 	if err != nil {
 		return err
 	}
 
 	if err = util.SetExec(exec); err != nil {
-		logrus.Errorf("Failed to initialize exec helper: %v", err)
-		return err
-	}
-
-	pidfile := ctx.String("pidfile")
-	if pidfile != "" {
-		defer delPidfile(pidfile)
-		err = setupPIDFile(pidfile)
-		if err != nil {
-			return err
-		}
+		return fmt.Errorf("failed to initialize exec helper: %v", err)
 	}
 
 	clientset, err := util.NewClientset(&config.Kubernetes)
 	if err != nil {
-		panic(err.Error())
+		return err
 	}
 
 	// create factory and start the controllers asked for
 	stopChan := make(chan struct{})
 	factory, err := factory.NewWatchFactory(clientset, stopChan)
 	if err != nil {
-		panic(err.Error())
+		return err
 	}
 
 	master := ctx.String("init-master")
 	node := ctx.String("init-node")
-	clusterController := ovncluster.NewClusterController(clientset, factory)
 
 	cleanupNode := ctx.String("cleanup-node")
 	if cleanupNode != "" {
 		if master != "" || node != "" {
-			panic("Cannot specify cleanup-node together with 'init-node or 'init-master'.")
+			return fmt.Errorf("cannot specify cleanup-node together with 'init-node or 'init-master'")
 		}
 
-		err = clusterController.CleanupClusterNode(cleanupNode)
-		if err != nil {
-			logrus.Errorf(err.Error())
-			panic(err.Error())
+		if err = ovncluster.CleanupClusterNode(cleanupNode); err != nil {
+			return err
 		}
 		return nil
 	}
 
-	if master != "" || node != "" {
-		if master != "" {
-			if runtime.GOOS == "windows" {
-				panic("Windows is not supported as master node")
-			}
-			// run the cluster controller to init the master
-			err := clusterController.StartClusterMaster(master)
-			if err != nil {
-				logrus.Errorf(err.Error())
-				panic(err.Error())
-			}
-			ovnController := ovn.NewOvnController(clientset, factory)
-			if err := ovnController.Run(); err != nil {
-				logrus.Errorf(err.Error())
-				panic(err.Error())
-			}
-		}
-
-		if node != "" {
-			if config.Kubernetes.Token == "" {
-				panic("Cannot initialize node without service account 'token'. Please provide one with --k8s-token argument")
-			}
-
-			err := clusterController.StartClusterNode(node)
-			if err != nil {
-				logrus.Errorf(err.Error())
-				panic(err.Error())
-			}
-		}
-
-		// run forever
-		select {}
+	if master == "" && node == "" {
+		return fmt.Errorf("need to run ovnkube in either master and/or node mode")
 	}
 
-	return fmt.Errorf("need to run ovnkube in either master and/or node mode")
+	// start the prometheus server
+	if config.Kubernetes.MetricsBindAddress != "" {
+		util.StartMetricsServer(config.Kubernetes.MetricsBindAddress, config.Kubernetes.MetricsEnablePprof)
+	}
+
+	// Set up a watch on our config file; if it changes, we exit -
+	// (we don't have the ability to dynamically reload config changes).
+	if err := watchForChanges(configFile); err != nil {
+		return fmt.Errorf("unable to setup configuration watch: %v", err)
+	}
+
+	enableHybridOverlay := ctx.Bool("enable-hybrid-overlay")
+	if enableHybridOverlay {
+		// Since the third address of every cluster subnet is reserved for
+		// the hybrid overlay, only allow enabling it for HostSubnets that
+		// are a /24 or larger.
+		for _, clusterEntry := range config.Default.ClusterSubnets {
+			if clusterEntry.HostSubnetLength > 24 {
+				return fmt.Errorf("hybrid overlay cannot be used with" +
+					" host subnet prefixes smaller than /24.")
+			}
+		}
+	}
+
+	if master != "" {
+		if runtime.GOOS == "windows" {
+			return fmt.Errorf("Windows is not supported as master node")
+		}
+
+		ovn.RegisterMetrics()
+
+		var hybridOverlayClusterSubnets []config.CIDRNetworkEntry
+		if enableHybridOverlay {
+			hybridOverlayClusterSubnets, err = hocontroller.GetHybridOverlayClusterSubnets(ctx)
+			if err != nil {
+				return err
+			}
+		}
+
+		// run the HA master controller to init the master
+		ovnHAController := ovn.NewHAMasterController(clientset, factory, master, stopChan, hybridOverlayClusterSubnets)
+		if err := ovnHAController.StartHAMasterController(); err != nil {
+			return err
+		}
+	}
+
+	if node != "" {
+		if config.Kubernetes.Token == "" {
+			return fmt.Errorf("cannot initialize node without service account 'token'. Please provide one with --k8s-token argument")
+		}
+
+		clusterController := ovncluster.NewClusterController(clientset, factory)
+		if err := clusterController.StartClusterNode(node); err != nil {
+			return err
+		}
+	}
+
+	if enableHybridOverlay {
+		if err := hocontroller.StartHybridOverlay(ctx, master != "", node, clientset, factory); err != nil {
+			return err
+		}
+	}
+
+	// run forever
+	select {}
+}
+
+// watchForChanges exits if the configuration file changed.
+func watchForChanges(configPath string) error {
+	if configPath == "" {
+		return nil
+	}
+	configPath, err := filepath.Abs(configPath)
+	if err != nil {
+		return err
+	}
+
+	watcher, err := fsnotify.NewWatcher()
+	if err != nil {
+		return err
+	}
+	defer watcher.Close()
+
+	go func() {
+		for {
+			select {
+			case event, ok := <-watcher.Events:
+				if !ok {
+					return
+				}
+				if event.Op&fsnotify.Write == fsnotify.Write {
+					logrus.Infof("Configuration file %s changed, exiting...", event.Name)
+					os.Exit(0)
+					return
+				}
+			case err, ok := <-watcher.Errors:
+				if !ok {
+					return
+				}
+				logrus.Errorf("fsnotify error %v", err)
+			}
+		}
+	}()
+
+	// Watch all symlinks for changes
+	p := configPath
+	maxdepth := 100
+	for depth := 0; depth < maxdepth; depth++ {
+		if err := watcher.Add(p); err != nil {
+			return err
+		}
+		logrus.Infof("Watching config file %s for changes", p)
+
+		stat, err := os.Lstat(p)
+		if err != nil {
+			return err
+		}
+
+		// configmaps are usually symlinks
+		if stat.Mode()&os.ModeSymlink > 0 {
+			p, err = filepath.EvalSymlinks(p)
+			if err != nil {
+				return err
+			}
+		} else {
+			break
+		}
+	}
+
+	return nil
 }
