@@ -4,13 +4,16 @@ package cluster
 
 import (
 	"fmt"
-	"k8s.io/client-go/tools/cache"
+	"net"
 	"reflect"
 	"strings"
 
+	"k8s.io/client-go/tools/cache"
+
 	"github.com/coreos/go-iptables/iptables"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/config"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/factory"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/ovn"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 	"github.com/sirupsen/logrus"
 
@@ -18,10 +21,16 @@ import (
 )
 
 const (
-	localnetGatewayIP            = "169.254.33.2/24"
-	localnetGatewayNextHop       = "169.254.33.1"
-	localnetGatewayNextHopSubnet = "169.254.33.1/24"
-	iptableNodePortChain         = "OVN-KUBE-NODEPORT"
+	v4localnetGatewayIP            = "169.254.33.2/24"
+	v4localnetGatewayNextHop       = "169.254.33.1"
+	v4localnetGatewayNextHopSubnet = "169.254.33.1/24"
+	v6localnetGatewayIP            = "fd99::2/64"
+	v6localnetGatewayNextHop       = "fd99::1"
+	v6localnetGatewayNextHopSubnet = "fd99::1/64"
+	// fixed MAC address for the br-nexthop interface. the last 4 hex bytes
+	// translates to the br-nexthop's IP address
+	localnetGatewayNextHopMac = "00:00:a9:fe:21:01"
+	iptableNodePortChain      = "OVN-KUBE-NODEPORT"
 )
 
 type iptRule struct {
@@ -30,6 +39,27 @@ type iptRule struct {
 	args  []string
 }
 
+func localnetGatewayIP() string {
+	if config.IPv6Mode {
+		return v6localnetGatewayIP
+	}
+	return v4localnetGatewayIP
+}
+
+func localnetGatewayNextHop() string {
+	if config.IPv6Mode {
+		return v6localnetGatewayNextHop
+	}
+	return v4localnetGatewayNextHop
+}
+
+func localnetGatewayNextHopSubnet() string {
+	if config.IPv6Mode {
+		return v6localnetGatewayNextHopSubnet
+	}
+	return v4localnetGatewayNextHopSubnet
+}
+
 func ensureChain(ipt util.IPTablesHelper, table, chain string) error {
 	chains, err := ipt.ListChains(table)
 	if err != nil {
@@ -106,11 +136,11 @@ func localnetGatewayNAT(ipt util.IPTablesHelper, ifname, ip string) error {
 	return addIptRules(ipt, rules)
 }
 
-func initLocalnetGateway(nodeName string, clusterIPSubnet []string,
-	subnet string, wf *factory.WatchFactory) error {
-	ipt, err := util.GetIPTablesHelper(iptables.ProtocolIPv4)
+func initLocalnetGateway(nodeName string,
+	subnet string, wf *factory.WatchFactory) (map[string]map[string]string, error) {
+	ipt, err := localnetIPTablesHelper()
 	if err != nil {
-		return fmt.Errorf("failed to initialize iptables: %v", err)
+		return nil, err
 	}
 
 	// Create a localnet OVS bridge.
@@ -118,67 +148,88 @@ func initLocalnetGateway(nodeName string, clusterIPSubnet []string,
 	_, stderr, err := util.RunOVSVsctl("--may-exist", "add-br",
 		localnetBridgeName)
 	if err != nil {
-		return fmt.Errorf("Failed to create localnet bridge %s"+
+		return nil, fmt.Errorf("Failed to create localnet bridge %s"+
 			", stderr:%s (%v)", localnetBridgeName, stderr, err)
 	}
 
+	ifaceID, macAddress, err := bridgedGatewayNodeSetup(nodeName, localnetBridgeName, localnetBridgeName, true)
+	if err != nil {
+		return nil, fmt.Errorf("failed to set up shared interface gateway: %v", err)
+	}
+
 	_, _, err = util.RunIP("link", "set", localnetBridgeName, "up")
 	if err != nil {
-		return fmt.Errorf("failed to up %s (%v)", localnetBridgeName, err)
+		return nil, fmt.Errorf("failed to up %s (%v)", localnetBridgeName, err)
 	}
 
 	// Create a localnet bridge nexthop
 	localnetBridgeNextHop := "br-nexthop"
-	_, stderr, err = util.RunOVSVsctl("--may-exist", "add-port",
-		localnetBridgeName, localnetBridgeNextHop, "--", "set",
-		"interface", localnetBridgeNextHop, "type=internal")
+	_, stderr, err = util.RunOVSVsctl(
+		"--may-exist", "add-port", localnetBridgeName, localnetBridgeNextHop,
+		"--", "set", "interface", localnetBridgeNextHop, "type=internal",
+		"mtu_request="+fmt.Sprintf("%d", config.Default.MTU),
+		fmt.Sprintf("mac=%s", strings.ReplaceAll(localnetGatewayNextHopMac, ":", "\\:")))
 	if err != nil {
-		return fmt.Errorf("Failed to create localnet bridge next hop %s"+
+		return nil, fmt.Errorf("Failed to create localnet bridge next hop %s"+
 			", stderr:%s (%v)", localnetBridgeNextHop, stderr, err)
 	}
 	_, _, err = util.RunIP("link", "set", localnetBridgeNextHop, "up")
 	if err != nil {
-		return fmt.Errorf("failed to up %s (%v)", localnetBridgeNextHop, err)
+		return nil, fmt.Errorf("failed to up %s (%v)", localnetBridgeNextHop, err)
 	}
 
 	// Flush IPv4 address of localnetBridgeNextHop.
 	_, _, err = util.RunIP("addr", "flush", "dev", localnetBridgeNextHop)
 	if err != nil {
-		return fmt.Errorf("failed to flush ip address of %s (%v)",
+		return nil, fmt.Errorf("failed to flush ip address of %s (%v)",
 			localnetBridgeNextHop, err)
 	}
 
 	// Set localnetBridgeNextHop with an IP address.
 	_, _, err = util.RunIP("addr", "add",
-		localnetGatewayNextHopSubnet,
+		localnetGatewayNextHopSubnet(),
 		"dev", localnetBridgeNextHop)
 	if err != nil {
-		return fmt.Errorf("failed to assign ip address to %s (%v)",
+		return nil, fmt.Errorf("failed to assign ip address to %s (%v)",
 			localnetBridgeNextHop, err)
 	}
 
-	ifaceID, macAddress, err := bridgedGatewayNodeSetup(nodeName, localnetBridgeName)
-	if err != nil {
-		return fmt.Errorf("failed to set up shared interface gateway: %v", err)
+	l3GatewayConfig := map[string]string{
+		ovn.OvnNodeGatewayMode:       string(config.Gateway.Mode),
+		ovn.OvnNodeGatewayVlanID:     fmt.Sprintf("%d", config.Gateway.VLANID),
+		ovn.OvnNodeGatewayIfaceID:    ifaceID,
+		ovn.OvnNodeGatewayMacAddress: macAddress,
+		ovn.OvnNodeGatewayIP:         localnetGatewayIP(),
+		ovn.OvnNodeGatewayNextHop:    localnetGatewayNextHop(),
+		ovn.OvnNodePortEnable:        fmt.Sprintf("%t", config.Gateway.NodeportEnable),
+	}
+	annotations := map[string]map[string]string{
+		ovn.OvnDefaultNetworkGateway: l3GatewayConfig,
 	}
 
-	err = util.GatewayInit(clusterIPSubnet, nodeName, ifaceID, localnetGatewayIP,
-		macAddress, localnetGatewayNextHop, subnet, true, nil)
-	if err != nil {
-		return fmt.Errorf("failed to localnet gateway: %v", err)
+	if config.IPv6Mode {
+		// TODO - IPv6 hack ... for some reason neighbor discovery isn't working here, so hard code a
+		// MAC binding for the gateway IP address for now - need to debug this further
+		_, _, _ = util.RunIP("-6", "neigh", "del", "fd99::2", "dev", "br-nexthop")
+		stdout, stderr, err := util.RunIP("-6", "neigh", "add", "fd99::2", "dev", "br-nexthop", "lladdr", macAddress)
+		if err == nil {
+			logrus.Infof("Added MAC binding for fd99::2 on br-nexthop, stdout: '%s', stderr: '%s'", stdout, stderr)
+		} else {
+			logrus.Errorf("Error in adding MAC binding for fd99::2 on br-nexthop: %v", err)
+		}
 	}
 
-	err = localnetGatewayNAT(ipt, localnetBridgeNextHop, localnetGatewayIP)
+	err = localnetGatewayNAT(ipt, localnetBridgeNextHop, localnetGatewayIP())
 	if err != nil {
-		return fmt.Errorf("Failed to add NAT rules for localnet gateway (%v)",
+		return nil, fmt.Errorf("Failed to add NAT rules for localnet gateway (%v)",
 			err)
 	}
 
 	if config.Gateway.NodeportEnable {
-		return localnetNodePortWatcher(ipt, wf)
+		err = localnetNodePortWatcher(ipt, wf)
 	}
 
-	return nil
+	return annotations, err
 }
 
 func localnetIptRules(svc *kapi.Service) []iptRule {
@@ -190,15 +241,13 @@ func localnetIptRules(svc *kapi.Service) []iptRule {
 		}
 
 		nodePort := fmt.Sprintf("%d", svcPort.NodePort)
-		destination := strings.Split(localnetGatewayIP, "/")[0] + ":" + nodePort
+		destination := strings.Split(localnetGatewayIP(), "/")[0] + ":" + nodePort
 
 		rules = append(rules, iptRule{
 			table: "nat",
 			chain: iptableNodePortChain,
-			args: []string{
-				"-p", string(protocol), "--dport", nodePort,
-				"-j", "DNAT", "--to-destination", destination,
-			},
+			args: []string{"-p", string(protocol), "--dport", nodePort, "-j", "DNAT", "--to-destination",
+				net.JoinHostPort(strings.Split(localnetGatewayIP(), "/")[0], nodePort)},
 		})
 		rules = append(rules, iptRule{
 			table: "filter",
@@ -237,14 +286,29 @@ func localnetIptRules(svc *kapi.Service) []iptRule {
 	return rules
 }
 
+// localnetIPTablesHelper gets an IPTablesHelper for IPv4 or IPv6 as appropriate
+func localnetIPTablesHelper() (util.IPTablesHelper, error) {
+	var ipt util.IPTablesHelper
+	var err error
+	if config.IPv6Mode {
+		ipt, err = util.GetIPTablesHelper(iptables.ProtocolIPv6)
+	} else {
+		ipt, err = util.GetIPTablesHelper(iptables.ProtocolIPv4)
+	}
+	if err != nil {
+		return nil, fmt.Errorf("failed to initialize iptables: %v", err)
+	}
+	return ipt, nil
+}
+
 // AddService adds service and creates corresponding resources in OVN
 func localnetAddService(svc *kapi.Service) error {
 	if !util.ServiceTypeHasNodePort(svc) {
 		return nil
 	}
-	ipt, err := util.GetIPTablesHelper(iptables.ProtocolIPv4)
+	ipt, err := localnetIPTablesHelper()
 	if err != nil {
-		return fmt.Errorf("failed to initialize iptables: %v", err)
+		return err
 	}
 	rules := localnetIptRules(svc)
 	logrus.Debugf("Add rules %v for service %v", rules, svc.Name)
@@ -255,9 +319,9 @@ func localnetDeleteService(svc *kapi.Service) error {
 	if !util.ServiceTypeHasNodePort(svc) {
 		return nil
 	}
-	ipt, err := util.GetIPTablesHelper(iptables.ProtocolIPv4)
+	ipt, err := localnetIPTablesHelper()
 	if err != nil {
-		return fmt.Errorf("failed to initialize iptables: %v", err)
+		return err
 	}
 	rules := localnetIptRules(svc)
 	logrus.Debugf("Delete rules %v for service %v", rules, svc.Name)
@@ -327,6 +391,7 @@ func localnetNodePortWatcher(ipt util.IPTablesHelper, wf *factory.WatchFactory)
 	return err
 }
 
+// cleanupLocalnetGateway cleans up Localnet Gateway
 func cleanupLocalnetGateway() error {
 	// get bridgeName from ovn-bridge-mappings.
 	stdout, stderr, err := util.RunOVSVsctl("--if-exists", "get", "Open_vSwitch", ".",
