@@ -3,7 +3,9 @@ package cni
 import (
 	"net/http"
 
+	"github.com/containernetworking/cni/pkg/types/current"
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/cni/types"
+	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 )
 
 // serverRunDir is the default directory for CNIServer runtime files
@@ -13,6 +15,15 @@ const serverSocketName string = "ovn-cni-server.sock"
 const serverSocketPath string = serverRunDir + "/" + serverSocketName
 const serverTCPAddress string = "127.0.0.1:3996"
 
+// PodInterfaceInfo consists of interface info result from cni server if cni client configure's interface
+type PodInterfaceInfo struct {
+	util.PodAnnotation
+
+	MTU     int   `json:"mtu"`
+	Ingress int64 `json:"ingress"`
+	Egress  int64 `json:"egress"`
+}
+
 // Explicit type for CNI commands the server handles
 type command string
 
@@ -33,6 +44,12 @@ type Request struct {
 	Config []byte `json:"config,omitempty"`
 }
 
+// Response sent to the OVN CNI plugin by the Server
+type Response struct {
+	Result    *current.Result
+	PodIFInfo *PodInterfaceInfo
+}
+
 // PodRequest structure built from Request which is passed to the
 // handler function given to the Server at creation time
 type PodRequest struct {
@@ -50,16 +67,6 @@ type PodRequest struct {
 	IfName string
 	// CNI conf obtained from stdin conf
 	CNIConf *types.NetConf
-	// Channel for returning the operation result to the Server
-	Result chan *PodResult
-}
-
-// PodResult of a PodRequest sent through the PodRequest's Result channel.
-type PodResult struct {
-	// Response to be returned to the OVN CNI plugin on success
-	Response []byte
-	// Error to be returned to the OVN CNI plugin on failure
-	Err error
 }
 
 type cniRequestFunc func(request *PodRequest) ([]byte, error)
