@@ -2,14 +2,13 @@ package ovn
 
 import (
 	"fmt"
-	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/factory"
+	"strings"
+
 	"github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
 	"github.com/sirupsen/logrus"
 	kapi "k8s.io/api/core/v1"
 	knet "k8s.io/api/networking/v1"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/client-go/tools/cache"
-	"strings"
 )
 
 func (oc *Controller) syncNetworkPoliciesOld(networkPolicies []interface{}) {
@@ -30,9 +29,9 @@ func (oc *Controller) syncNetworkPoliciesOld(networkPolicies []interface{}) {
 		if policyName != "" &&
 			!expectedPolicies[namespaceName][policyName] {
 			// policy doesn't exist on k8s. Delete acl rules from OVN
-			oc.deleteAclsPolicyOld(namespaceName, policyName)
+			deleteAclsPolicyOld(namespaceName, policyName)
 			// delete the address sets for this policy from OVN
-			oc.deleteAddressSet(hashedAddressSet(addrSetName))
+			deleteAddressSet(hashedAddressSet(addrSetName))
 		}
 	})
 	if err != nil {
@@ -40,9 +39,8 @@ func (oc *Controller) syncNetworkPoliciesOld(networkPolicies []interface{}) {
 	}
 }
 
-func (oc *Controller) addACLAllowOld(namespace, policy, logicalSwitch,
-	logicalPort, match, l4Match string, ipBlockCidr bool, gressNum int,
-	policyType knet.PolicyType) {
+func addACLAllowOld(namespace, policy, logicalSwitch, logicalPort, match, l4Match string,
+	ipBlockCidr bool, gressNum int, policyType knet.PolicyType) {
 	var direction, action string
 	direction = toLport
 	if policyType == knet.PolicyTypeIngress {
@@ -93,8 +91,8 @@ func (oc *Controller) addACLAllowOld(namespace, policy, logicalSwitch,
 	}
 }
 
-func (oc *Controller) modifyACLAllowOld(namespace, policy, logicalPort,
-	oldMatch string, newMatch string, gressNum int, policyType knet.PolicyType) {
+func modifyACLAllowOld(namespace, policy, logicalPort, oldMatch string, newMatch string,
+	gressNum int, policyType knet.PolicyType) {
 	uuid, stderr, err := util.RunOVNNbctl("--data=bare", "--no-heading",
 		"--columns=_uuid", "find", "ACL", oldMatch,
 		fmt.Sprintf("external-ids:namespace=%s", namespace),
@@ -112,7 +110,7 @@ func (oc *Controller) modifyACLAllowOld(namespace, policy, logicalPort,
 	if uuid != "" {
 		// We already have an ACL. We will update it.
 		_, stderr, err = util.RunOVNNbctl("set", "acl", uuid,
-			fmt.Sprintf("%s", newMatch))
+			newMatch)
 		if err != nil {
 			logrus.Errorf("failed to modify the allow-from rule for "+
 				"namespace=%s, logical_port=%s, stderr: %q (%v)",
@@ -122,9 +120,8 @@ func (oc *Controller) modifyACLAllowOld(namespace, policy, logicalPort,
 	}
 }
 
-func (oc *Controller) deleteACLAllowOld(namespace, policy, logicalSwitch,
-	logicalPort, match, l4Match string, ipBlockCidr bool, gressNum int,
-	policyType knet.PolicyType) {
+func deleteACLAllowOld(namespace, policy, logicalSwitch, logicalPort, match, l4Match string,
+	ipBlockCidr bool, gressNum int, policyType knet.PolicyType) {
 	uuid, stderr, err := util.RunOVNNbctl("--data=bare", "--no-heading",
 		"--columns=_uuid", "find", "ACL",
 		fmt.Sprintf("external-ids:l4Match=\"%s\"", l4Match),
@@ -157,8 +154,8 @@ func (oc *Controller) deleteACLAllowOld(namespace, policy, logicalSwitch,
 	}
 }
 
-func (oc *Controller) addIPBlockACLDenyOld(namespace, policy, logicalSwitch,
-	logicalPort, except, priority string, policyType knet.PolicyType) {
+func addIPBlockACLDenyOld(namespace, policy, logicalSwitch, logicalPort, except, priority string,
+	policyType knet.PolicyType) {
 	var match, l3Match, direction, lportMatch string
 	direction = toLport
 	if policyType == knet.PolicyTypeIngress {
@@ -203,11 +200,10 @@ func (oc *Controller) addIPBlockACLDenyOld(namespace, policy, logicalSwitch,
 		logrus.Errorf("error executing create ACL command, stderr: %q, %+v",
 			stderr, err)
 	}
-	return
 }
 
-func (oc *Controller) deleteIPBlockACLDenyOld(namespace, policy,
-	logicalSwitch, logicalPort, except string, policyType knet.PolicyType) {
+func deleteIPBlockACLDenyOld(namespace, policy, logicalSwitch, logicalPort, except string,
+	policyType knet.PolicyType) {
 	var match, lportMatch, l3Match string
 	if policyType == knet.PolicyTypeIngress {
 		lportMatch = fmt.Sprintf("outport == \\\"%s\\\"", logicalPort)
@@ -245,11 +241,9 @@ func (oc *Controller) deleteIPBlockACLDenyOld(namespace, policy,
 			namespace, logicalPort, stderr, err)
 		return
 	}
-	return
 }
 
-func (oc *Controller) addACLDenyOld(namespace, logicalSwitch, logicalPort,
-	priority string, policyType knet.PolicyType) {
+func addACLDenyOld(namespace, logicalSwitch, logicalPort, priority string, policyType knet.PolicyType) {
 	var match, direction string
 	direction = toLport
 	if policyType == knet.PolicyTypeIngress {
@@ -288,11 +282,9 @@ func (oc *Controller) addACLDenyOld(namespace, logicalSwitch, logicalPort,
 		logrus.Errorf("error executing create ACL command, stderr: %q, %+v",
 			stderr, err)
 	}
-	return
 }
 
-func (oc *Controller) deleteACLDenyOld(namespace, logicalSwitch, logicalPort string,
-	policyType knet.PolicyType) {
+func deleteACLDenyOld(namespace, logicalSwitch, logicalPort string, policyType knet.PolicyType) {
 	var match string
 	if policyType == knet.PolicyTypeIngress {
 		match = fmt.Sprintf("match=\"outport == \\\"%s\\\"\"", logicalPort)
@@ -325,10 +317,9 @@ func (oc *Controller) deleteACLDenyOld(namespace, logicalSwitch, logicalPort str
 			namespace, logicalPort, stderr, err)
 		return
 	}
-	return
 }
 
-func (oc *Controller) deleteAclsPolicyOld(namespace, policy string) {
+func deleteAclsPolicyOld(namespace, policy string) {
 	uuids, stderr, err := util.RunOVNNbctl("--data=bare", "--no-heading",
 		"--columns=_uuid", "find", "ACL",
 		fmt.Sprintf("external-ids:namespace=%s", namespace),
@@ -373,10 +364,9 @@ func (oc *Controller) deleteAclsPolicyOld(namespace, policy string) {
 	}
 }
 
-func (oc *Controller) localPodAddOrDelACLOld(addDel string,
-	policy *knet.NetworkPolicy, pod *kapi.Pod, gress *gressPolicy,
-	logicalSwitch string) {
-	logicalPort := fmt.Sprintf("%s_%s", pod.Namespace, pod.Name)
+func localPodAddOrDelACLOld(addDel string, policy *knet.NetworkPolicy, pod *kapi.Pod,
+	gress *gressPolicy, logicalSwitch string) {
+	logicalPort := podLogicalPortName(pod)
 	l3Match := gress.getL3MatchFromAddressSet()
 
 	var lportMatch, cidrMatch string
@@ -391,10 +381,10 @@ func (oc *Controller) localPodAddOrDelACLOld(addDel string,
 	if len(gress.ipBlockCidr) > 0 && len(gress.ipBlockExcept) > 0 {
 		except := fmt.Sprintf("{%s}", strings.Join(gress.ipBlockExcept, ", "))
 		if addDel == addACL {
-			oc.addIPBlockACLDenyOld(policy.Namespace, policy.Name, logicalSwitch,
+			addIPBlockACLDenyOld(policy.Namespace, policy.Name, logicalSwitch,
 				logicalPort, except, ipBlockDenyPriority, gress.policyType)
 		} else {
-			oc.deleteIPBlockACLDenyOld(policy.Namespace, policy.Name,
+			deleteIPBlockACLDenyOld(policy.Namespace, policy.Name,
 				logicalSwitch, logicalPort, except, gress.policyType)
 		}
 	}
@@ -408,22 +398,22 @@ func (oc *Controller) localPodAddOrDelACLOld(addDel string,
 			if len(gress.ipBlockCidr) > 0 {
 				// Add ACL allow rule for IPBlock CIDR
 				cidrMatch = gress.getMatchFromIPBlock(lportMatch, l4Match)
-				oc.addACLAllowOld(policy.Namespace, policy.Name,
+				addACLAllowOld(policy.Namespace, policy.Name,
 					logicalSwitch, logicalPort, cidrMatch, l4Match,
 					true, gress.idx, gress.policyType)
 			}
-			oc.addACLAllowOld(policy.Namespace, policy.Name,
+			addACLAllowOld(policy.Namespace, policy.Name,
 				logicalSwitch, logicalPort, match, l4Match,
 				false, gress.idx, gress.policyType)
 		} else {
 			if len(gress.ipBlockCidr) > 0 {
 				// Delete ACL allow rule for IPBlock CIDR
 				cidrMatch = gress.getMatchFromIPBlock(lportMatch, l4Match)
-				oc.deleteACLAllowOld(policy.Namespace, policy.Name,
+				deleteACLAllowOld(policy.Namespace, policy.Name,
 					logicalSwitch, logicalPort, cidrMatch, l4Match,
 					true, gress.idx, gress.policyType)
 			}
-			oc.deleteACLAllowOld(policy.Namespace, policy.Name,
+			deleteACLAllowOld(policy.Namespace, policy.Name,
 				logicalSwitch, logicalPort, match, l4Match,
 				false, gress.idx, gress.policyType)
 		}
@@ -439,22 +429,22 @@ func (oc *Controller) localPodAddOrDelACLOld(addDel string,
 			if len(gress.ipBlockCidr) > 0 {
 				// Add ACL allow rule for IPBlock CIDR
 				cidrMatch = gress.getMatchFromIPBlock(lportMatch, l4Match)
-				oc.addACLAllowOld(policy.Namespace, policy.Name,
+				addACLAllowOld(policy.Namespace, policy.Name,
 					logicalSwitch, logicalPort, cidrMatch, l4Match,
 					true, gress.idx, gress.policyType)
 			}
-			oc.addACLAllowOld(policy.Namespace, policy.Name,
+			addACLAllowOld(policy.Namespace, policy.Name,
 				pod.Spec.NodeName, logicalPort, match, l4Match,
 				false, gress.idx, gress.policyType)
 		} else {
 			if len(gress.ipBlockCidr) > 0 {
 				// Delete ACL allow rule for IPBlock CIDR
 				cidrMatch = gress.getMatchFromIPBlock(lportMatch, l4Match)
-				oc.deleteACLAllowOld(policy.Namespace, policy.Name,
+				deleteACLAllowOld(policy.Namespace, policy.Name,
 					logicalSwitch, logicalPort, cidrMatch, l4Match,
 					true, gress.idx, gress.policyType)
 			}
-			oc.deleteACLAllowOld(policy.Namespace, policy.Name,
+			deleteACLAllowOld(policy.Namespace, policy.Name,
 				pod.Spec.NodeName, logicalPort, match, l4Match,
 				false, gress.idx, gress.policyType)
 		}
@@ -464,7 +454,6 @@ func (oc *Controller) localPodAddOrDelACLOld(addDel string,
 func (oc *Controller) localPodAddDefaultDenyOld(
 	policy *knet.NetworkPolicy,
 	logicalPort, logicalSwitch string) {
-
 	oc.lspMutex.Lock()
 	// Default deny rule.
 	// 1. Any pod that matches a network policy should get a default
@@ -480,7 +469,7 @@ func (oc *Controller) localPodAddDefaultDenyOld(
 	// Handle condition 1 above.
 	if !(len(policy.Spec.PolicyTypes) == 1 && policy.Spec.PolicyTypes[0] == knet.PolicyTypeEgress) {
 		if oc.lspIngressDenyCache[logicalPort] == 0 {
-			oc.addACLDenyOld(policy.Namespace, logicalSwitch, logicalPort,
+			addACLDenyOld(policy.Namespace, logicalSwitch, logicalPort,
 				defaultDenyPriority, knet.PolicyTypeIngress)
 		}
 		oc.lspIngressDenyCache[logicalPort]++
@@ -490,7 +479,7 @@ func (oc *Controller) localPodAddDefaultDenyOld(
 	if (len(policy.Spec.PolicyTypes) == 1 && policy.Spec.PolicyTypes[0] == knet.PolicyTypeEgress) ||
 		len(policy.Spec.Egress) > 0 || len(policy.Spec.PolicyTypes) == 2 {
 		if oc.lspEgressDenyCache[logicalPort] == 0 {
-			oc.addACLDenyOld(policy.Namespace, logicalSwitch, logicalPort,
+			addACLDenyOld(policy.Namespace, logicalSwitch, logicalPort,
 				defaultDenyPriority, knet.PolicyTypeEgress)
 		}
 		oc.lspEgressDenyCache[logicalPort]++
@@ -507,7 +496,7 @@ func (oc *Controller) localPodDelDefaultDenyOld(
 		if oc.lspIngressDenyCache[logicalPort] > 0 {
 			oc.lspIngressDenyCache[logicalPort]--
 			if oc.lspIngressDenyCache[logicalPort] == 0 {
-				oc.deleteACLDenyOld(policy.Namespace, logicalSwitch,
+				deleteACLDenyOld(policy.Namespace, logicalSwitch,
 					logicalPort, knet.PolicyTypeIngress)
 			}
 		}
@@ -518,7 +507,7 @@ func (oc *Controller) localPodDelDefaultDenyOld(
 		if oc.lspEgressDenyCache[logicalPort] > 0 {
 			oc.lspEgressDenyCache[logicalPort]--
 			if oc.lspEgressDenyCache[logicalPort] == 0 {
-				oc.deleteACLDenyOld(policy.Namespace, logicalSwitch,
+				deleteACLDenyOld(policy.Namespace, logicalSwitch,
 					logicalPort, knet.PolicyTypeEgress)
 			}
 		}
@@ -531,8 +520,7 @@ func (oc *Controller) handleLocalPodSelectorAddFuncOld(
 	obj interface{}) {
 	pod := obj.(*kapi.Pod)
 
-	ipAddress := oc.getIPFromOvnAnnotation(pod.Annotations["ovn"])
-	if ipAddress == "" {
+	if _, err := util.UnmarshalPodAnnotation(pod.Annotations); err != nil {
 		return
 	}
 
@@ -542,7 +530,7 @@ func (oc *Controller) handleLocalPodSelectorAddFuncOld(
 	}
 
 	// Get the logical port name.
-	logicalPort := fmt.Sprintf("%s_%s", pod.Namespace, pod.Name)
+	logicalPort := podLogicalPortName(pod)
 
 	np.Lock()
 	defer np.Unlock()
@@ -559,11 +547,11 @@ func (oc *Controller) handleLocalPodSelectorAddFuncOld(
 
 	// For each ingress rule, add a ACL
 	for _, ingress := range np.ingressPolicies {
-		oc.localPodAddOrDelACLOld(addACL, policy, pod, ingress, logicalSwitch)
+		localPodAddOrDelACLOld(addACL, policy, pod, ingress, logicalSwitch)
 	}
 	// For each egress rule, add a ACL
 	for _, egress := range np.egressPolicies {
-		oc.localPodAddOrDelACLOld(addACL, policy, pod, egress, logicalSwitch)
+		localPodAddOrDelACLOld(addACL, policy, pod, egress, logicalSwitch)
 	}
 
 	np.localPods[logicalPort] = true
@@ -580,7 +568,7 @@ func (oc *Controller) handleLocalPodSelectorDelFuncOld(
 	}
 
 	// Get the logical port name.
-	logicalPort := fmt.Sprintf("%s_%s", pod.Namespace, pod.Name)
+	logicalPort := podLogicalPortName(pod)
 
 	np.Lock()
 	defer np.Unlock()
@@ -595,13 +583,18 @@ func (oc *Controller) handleLocalPodSelectorDelFuncOld(
 	delete(np.localPods, logicalPort)
 	oc.localPodDelDefaultDenyOld(policy, logicalPort, logicalSwitch)
 
+	oc.lspMutex.Lock()
+	delete(oc.lspIngressDenyCache, logicalPort)
+	delete(oc.lspEgressDenyCache, logicalPort)
+	oc.lspMutex.Unlock()
+
 	// For each ingress rule, remove the ACL
 	for _, ingress := range np.ingressPolicies {
-		oc.localPodAddOrDelACLOld(deleteACL, policy, pod, ingress, logicalSwitch)
+		localPodAddOrDelACLOld(deleteACL, policy, pod, ingress, logicalSwitch)
 	}
 	// For each egress rule, remove the ACL
 	for _, egress := range np.egressPolicies {
-		oc.localPodAddOrDelACLOld(deleteACL, policy, pod, egress, logicalSwitch)
+		localPodAddOrDelACLOld(deleteACL, policy, pod, egress, logicalSwitch)
 	}
 }
 
@@ -631,153 +624,6 @@ func (oc *Controller) handleLocalPodSelectorOld(
 
 }
 
-func (oc *Controller) handlePeerPodSelectorAddUpdateOld(
-	policy *knet.NetworkPolicy, np *namespacePolicy,
-	addressMap map[string]bool, addressSet string,
-	obj interface{}) {
-
-	pod := obj.(*kapi.Pod)
-	ipAddress := oc.getIPFromOvnAnnotation(pod.Annotations["ovn"])
-	if ipAddress == "" || addressMap[ipAddress] {
-		return
-	}
-
-	np.Lock()
-	defer np.Unlock()
-	if np.deleted {
-		return
-	}
-
-	addressMap[ipAddress] = true
-	addresses := make([]string, 0, len(addressMap))
-	for k := range addressMap {
-		addresses = append(addresses, k)
-	}
-	oc.setAddressSet(addressSet, addresses)
-}
-
-func (oc *Controller) handlePeerPodSelectorDeleteOld(
-	policy *knet.NetworkPolicy, np *namespacePolicy,
-	addressMap map[string]bool, addressSet string,
-	obj interface{}) {
-
-	pod := obj.(*kapi.Pod)
-
-	ipAddress := oc.getIPFromOvnAnnotation(pod.Annotations["ovn"])
-	if ipAddress == "" {
-		return
-	}
-
-	np.Lock()
-	defer np.Unlock()
-	if np.deleted {
-		return
-	}
-
-	if !addressMap[ipAddress] {
-		return
-	}
-
-	delete(addressMap, ipAddress)
-
-	addresses := make([]string, 0, len(addressMap))
-	for k := range addressMap {
-		addresses = append(addresses, k)
-	}
-	oc.setAddressSet(addressSet, addresses)
-
-}
-
-func (oc *Controller) handlePeerPodSelectorOld(
-	policy *knet.NetworkPolicy, podSelector *metav1.LabelSelector,
-	addressSet string, addressMap map[string]bool, np *namespacePolicy) {
-
-	h, err := oc.watchFactory.AddFilteredPodHandler(policy.Namespace,
-		podSelector,
-		cache.ResourceEventHandlerFuncs{
-			AddFunc: func(obj interface{}) {
-				oc.handlePeerPodSelectorAddUpdateOld(policy, np, addressMap, addressSet, obj)
-			},
-			DeleteFunc: func(obj interface{}) {
-				oc.handlePeerPodSelectorDeleteOld(policy, np, addressMap, addressSet, obj)
-			},
-			UpdateFunc: func(oldObj, newObj interface{}) {
-				oc.handlePeerPodSelectorAddUpdateOld(policy, np, addressMap, addressSet, newObj)
-			},
-		}, nil)
-	if err != nil {
-		logrus.Errorf("error watching peer pods for policy %s in namespace %s: %v",
-			policy.Name, policy.Namespace, err)
-		return
-	}
-
-	np.podHandlerList = append(np.podHandlerList, h)
-
-}
-
-func (oc *Controller) handlePeerNamespaceAndPodSelectorOld(
-	policy *knet.NetworkPolicy,
-	namespaceSelector *metav1.LabelSelector,
-	podSelector *metav1.LabelSelector,
-	addressSet string,
-	addressMap map[string]bool,
-	gress *gressPolicy,
-	np *namespacePolicy) {
-
-	namespaceHandler, err := oc.watchFactory.AddFilteredNamespaceHandler("",
-		namespaceSelector,
-		cache.ResourceEventHandlerFuncs{
-			AddFunc: func(obj interface{}) {
-				namespace := obj.(*kapi.Namespace)
-				np.Lock()
-				alreadyDeleted := np.deleted
-				np.Unlock()
-				if alreadyDeleted {
-					return
-				}
-
-				// The AddFilteredPodHandler call might call handlePeerPodSelectorAddUpdate
-				// on existing pods so we can't be holding the lock at this point
-				podHandler, err := oc.watchFactory.AddFilteredPodHandler(namespace.Name,
-					podSelector,
-					cache.ResourceEventHandlerFuncs{
-						AddFunc: func(obj interface{}) {
-							oc.handlePeerPodSelectorAddUpdateOld(policy, np, addressMap, addressSet, obj)
-						},
-						DeleteFunc: func(obj interface{}) {
-							oc.handlePeerPodSelectorDeleteOld(policy, np, addressMap, addressSet, obj)
-						},
-						UpdateFunc: func(oldObj, newObj interface{}) {
-							oc.handlePeerPodSelectorAddUpdateOld(policy, np, addressMap, addressSet, newObj)
-						},
-					}, nil)
-				if err != nil {
-					logrus.Errorf("error watching pods in namespace %s for policy %s: %v", namespace.Name, policy.Name, err)
-					return
-				}
-				np.Lock()
-				defer np.Unlock()
-				if np.deleted {
-					_ = oc.watchFactory.RemovePodHandler(podHandler)
-					return
-				}
-				np.podHandlerList = append(np.podHandlerList, podHandler)
-			},
-			DeleteFunc: func(obj interface{}) {
-				return
-			},
-			UpdateFunc: func(oldObj, newObj interface{}) {
-				return
-			},
-		}, nil)
-	if err != nil {
-		logrus.Errorf("error watching namespaces for policy %s: %v",
-			policy.Name, err)
-		return
-	}
-	np.nsHandlerList = append(np.nsHandlerList, namespaceHandler)
-}
-
 func (oc *Controller) handlePeerNamespaceSelectorModifyOld(
 	gress *gressPolicy, np *namespacePolicy, oldl3Match, newl3Match string) {
 
@@ -793,7 +639,7 @@ func (oc *Controller) handlePeerNamespaceSelectorModifyOld(
 				lportMatch)
 			newMatch := fmt.Sprintf("match=\"%s && %s\"", newl3Match,
 				lportMatch)
-			oc.modifyACLAllowOld(np.namespace, np.name, logicalPort,
+			modifyACLAllowOld(np.namespace, np.name, logicalPort,
 				oldMatch, newMatch, gress.idx, gress.policyType)
 		}
 		for _, port := range gress.portPolicies {
@@ -805,63 +651,14 @@ func (oc *Controller) handlePeerNamespaceSelectorModifyOld(
 				oldl3Match, l4Match, lportMatch)
 			newMatch := fmt.Sprintf("match=\"%s && %s && %s\"",
 				newl3Match, l4Match, lportMatch)
-			oc.modifyACLAllowOld(np.namespace, np.name, logicalPort,
+			modifyACLAllowOld(np.namespace, np.name, logicalPort,
 				oldMatch, newMatch, gress.idx, gress.policyType)
 		}
 	}
 }
 
-func (oc *Controller) handlePeerNamespaceSelectorOld(
-	policy *knet.NetworkPolicy,
-	namespaceSelector *metav1.LabelSelector,
-	gress *gressPolicy, np *namespacePolicy) {
-
-	h, err := oc.watchFactory.AddFilteredNamespaceHandler("",
-		namespaceSelector,
-		cache.ResourceEventHandlerFuncs{
-			AddFunc: func(obj interface{}) {
-				namespace := obj.(*kapi.Namespace)
-				np.Lock()
-				defer np.Unlock()
-				if np.deleted {
-					return
-				}
-				hashedAddressSet := hashedAddressSet(namespace.Name)
-				oldL3Match, newL3Match, added := gress.addAddressSet(hashedAddressSet)
-				if added {
-					oc.handlePeerNamespaceSelectorModifyOld(gress,
-						np, oldL3Match, newL3Match)
-				}
-			},
-			DeleteFunc: func(obj interface{}) {
-				namespace := obj.(*kapi.Namespace)
-				np.Lock()
-				defer np.Unlock()
-				if np.deleted {
-					return
-				}
-				hashedAddressSet := hashedAddressSet(namespace.Name)
-				oldL3Match, newL3Match, removed := gress.delAddressSet(hashedAddressSet)
-				if removed {
-					oc.handlePeerNamespaceSelectorModifyOld(gress,
-						np, oldL3Match, newL3Match)
-				}
-			},
-			UpdateFunc: func(oldObj, newObj interface{}) {
-				return
-			},
-		}, nil)
-	if err != nil {
-		logrus.Errorf("error watching namespaces for policy %s: %v",
-			policy.Name, err)
-		return
-	}
-
-	np.nsHandlerList = append(np.nsHandlerList, h)
-
-}
-
-// AddNetworkPolicy adds network policy and create corresponding acl rules
+// addNetworkPolicyOld creates and applies OVN ACLs to pod logical switch
+// ports from Kubernetes NetworkPolicy objects without using OVN Port Groups
 func (oc *Controller) addNetworkPolicyOld(policy *knet.NetworkPolicy) {
 	logrus.Infof("Adding network policy %s in namespace %s", policy.Name,
 		policy.Namespace)
@@ -878,14 +675,7 @@ func (oc *Controller) addNetworkPolicyOld(policy *knet.NetworkPolicy) {
 		return
 	}
 
-	np := &namespacePolicy{}
-	np.name = policy.Name
-	np.namespace = policy.Namespace
-	np.ingressPolicies = make([]*gressPolicy, 0)
-	np.egressPolicies = make([]*gressPolicy, 0)
-	np.podHandlerList = make([]*factory.Handler, 0)
-	np.nsHandlerList = make([]*factory.Handler, 0)
-	np.localPods = make(map[string]bool)
+	np := NewNamespacePolicy(policy)
 
 	// Go through each ingress rule.  For each ingress rule, create an
 	// addressSet for the peer pods.
@@ -910,7 +700,7 @@ func (oc *Controller) addNetworkPolicyOld(policy *knet.NetworkPolicy) {
 				policy.Name, "ingress", i)
 
 			hashedLocalAddressSet = hashedAddressSet(localPeerPods)
-			oc.createAddressSet(localPeerPods, hashedLocalAddressSet, nil)
+			createAddressSet(localPeerPods, hashedLocalAddressSet, nil)
 			ingress.addAddressSet(hashedLocalAddressSet)
 		}
 
@@ -924,19 +714,20 @@ func (oc *Controller) addNetworkPolicyOld(policy *knet.NetworkPolicy) {
 				// For each rule that contains both peer namespace selector and
 				// peer pod selector, we create a watcher for each matching namespace
 				// that populates the addressSet
-				oc.handlePeerNamespaceAndPodSelectorOld(policy,
+				oc.handlePeerNamespaceAndPodSelector(policy, ingress,
 					fromJSON.NamespaceSelector, fromJSON.PodSelector,
-					hashedLocalAddressSet, peerPodAddressMap, ingress, np)
+					hashedLocalAddressSet, peerPodAddressMap, np)
 
 			} else if fromJSON.NamespaceSelector != nil {
 				// For each peer namespace selector, we create a watcher that
 				// populates ingress.peerAddressSets
-				oc.handlePeerNamespaceSelectorOld(policy,
-					fromJSON.NamespaceSelector, ingress, np)
+				oc.handlePeerNamespaceSelector(policy,
+					fromJSON.NamespaceSelector, ingress, np,
+					oc.handlePeerNamespaceSelectorModifyOld)
 			} else if fromJSON.PodSelector != nil {
 				// For each peer pod selector, we create a watcher that
 				// populates the addressSet
-				oc.handlePeerPodSelectorOld(policy, fromJSON.PodSelector,
+				oc.handlePeerPodSelector(policy, fromJSON.PodSelector,
 					hashedLocalAddressSet, peerPodAddressMap, np)
 			}
 		}
@@ -966,7 +757,7 @@ func (oc *Controller) addNetworkPolicyOld(policy *knet.NetworkPolicy) {
 				policy.Name, "egress", i)
 
 			hashedLocalAddressSet = hashedAddressSet(localPeerPods)
-			oc.createAddressSet(localPeerPods, hashedLocalAddressSet, nil)
+			createAddressSet(localPeerPods, hashedLocalAddressSet, nil)
 			egress.addAddressSet(hashedLocalAddressSet)
 		}
 
@@ -980,18 +771,19 @@ func (oc *Controller) addNetworkPolicyOld(policy *knet.NetworkPolicy) {
 				// For each rule that contains both peer namespace selector and
 				// peer pod selector, we create a watcher for each matching namespace
 				// that populates the addressSet
-				oc.handlePeerNamespaceAndPodSelectorOld(policy,
+				oc.handlePeerNamespaceAndPodSelector(policy, egress,
 					toJSON.NamespaceSelector, toJSON.PodSelector,
-					hashedLocalAddressSet, peerPodAddressMap, egress, np)
+					hashedLocalAddressSet, peerPodAddressMap, np)
 			} else if toJSON.NamespaceSelector != nil {
 				// For each peer namespace selector, we create a watcher that
 				// populates egress.peerAddressSets
-				oc.handlePeerNamespaceSelectorOld(policy,
-					toJSON.NamespaceSelector, egress, np)
+				oc.handlePeerNamespaceSelector(policy,
+					toJSON.NamespaceSelector, egress, np,
+					oc.handlePeerNamespaceSelectorModifyOld)
 			} else if toJSON.PodSelector != nil {
 				// For each peer pod selector, we create a watcher that
 				// populates the addressSet
-				oc.handlePeerPodSelectorOld(policy, toJSON.PodSelector,
+				oc.handlePeerPodSelector(policy, toJSON.PodSelector,
 					hashedLocalAddressSet, peerPodAddressMap, np)
 			}
 		}
@@ -1003,8 +795,6 @@ func (oc *Controller) addNetworkPolicyOld(policy *knet.NetworkPolicy) {
 	// For all the pods in the local namespace that this policy
 	// effects, add ACL rules.
 	oc.handleLocalPodSelectorOld(policy, np)
-
-	return
 }
 
 func (oc *Controller) getLogicalSwitchForLogicalPort(
@@ -1054,7 +844,7 @@ func (oc *Controller) deleteNetworkPolicyOld(
 		localPeerPods := fmt.Sprintf("%s.%s.%s.%d", policy.Namespace,
 			policy.Name, "ingress", i)
 		hashedAddressSet := hashedAddressSet(localPeerPods)
-		oc.deleteAddressSet(hashedAddressSet)
+		deleteAddressSet(hashedAddressSet)
 	}
 	// Go through each egress rule.  For each egress rule, delete the
 	// addressSet for the local peer pods.
@@ -1062,7 +852,7 @@ func (oc *Controller) deleteNetworkPolicyOld(
 		localPeerPods := fmt.Sprintf("%s.%s.%s.%d", policy.Namespace,
 			policy.Name, "egress", i)
 		hashedAddressSet := hashedAddressSet(localPeerPods)
-		oc.deleteAddressSet(hashedAddressSet)
+		deleteAddressSet(hashedAddressSet)
 	}
 
 	// We should now stop all the handlers go routines.
@@ -1076,7 +866,5 @@ func (oc *Controller) deleteNetworkPolicyOld(
 	oc.namespacePolicies[policy.Namespace][policy.Name] = nil
 
 	// We should now delete all the ACLs added by this network policy.
-	oc.deleteAclsPolicyOld(policy.Namespace, policy.Name)
-
-	return
+	deleteAclsPolicyOld(policy.Namespace, policy.Name)
 }
